'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _htmlWebpackPlugin = require('html-webpack-plugin');

var _htmlWebpackPlugin2 = _interopRequireDefault(_htmlWebpackPlugin);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _chunkManifestWebpackPlugin = require('chunk-manifest-webpack-plugin');

var _chunkManifestWebpackPlugin2 = _interopRequireDefault(_chunkManifestWebpackPlugin);

var _webpackChunkHash = require('webpack-chunk-hash');

var _webpackChunkHash2 = _interopRequireDefault(_webpackChunkHash);

var _uglifyjsWebpackPlugin = require('uglifyjs-webpack-plugin');

var _uglifyjsWebpackPlugin2 = _interopRequireDefault(_uglifyjsWebpackPlugin);

var _bundling = require('./../utils/bundling');

var _defaultConfig = require('./../defaultConfig');

var _defaultConfig2 = _interopRequireDefault(_defaultConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var isVendor = function isVendor(_ref) {
  var resource = _ref.resource;
  return resource && resource.indexOf('node_modules') >= 0 && resource.match(/\.js$/);
};

exports.default = function (_ref2) {
  var _ref2$context = _ref2.context,
      context = _ref2$context === undefined ? _defaultConfig2.default.context : _ref2$context,
      _ref2$distOutput = _ref2.distOutput,
      distOutput = _ref2$distOutput === undefined ? _defaultConfig2.default.distOutput : _ref2$distOutput,
      _ref2$jsPath = _ref2.jsPath,
      jsPath = _ref2$jsPath === undefined ? _defaultConfig2.default.jsPath : _ref2$jsPath,
      _ref2$cssPath = _ref2.cssPath,
      cssPath = _ref2$cssPath === undefined ? _defaultConfig2.default.cssPath : _ref2$cssPath,
      _ref2$devCssBundle = _ref2.devCssBundle,
      devCssBundle = _ref2$devCssBundle === undefined ? _defaultConfig2.default.devCssBundle : _ref2$devCssBundle,
      _ref2$prodCssBundle = _ref2.prodCssBundle,
      prodCssBundle = _ref2$prodCssBundle === undefined ? _defaultConfig2.default.prodCssBundle : _ref2$prodCssBundle,
      isDevEnvironment = _ref2.isDevEnvironment,
      _ref2$plugins = _ref2.plugins,
      plugins = _ref2$plugins === undefined ? [] : _ref2$plugins,
      routes = _ref2.routes;

  //  Gets all { bundle } values from the routeConfig
  var bundleNames = (0, _bundling.getBundlesByName)(routes);

  //  Environment plugins define how webpack should
  //  be bundled for both production and development.
  //  Things like hot-module-reloading aren't required for
  //  prod, as an example.
  var environmentPlugins = !isDevEnvironment ? [new _webpack2.default.HashedModuleIdsPlugin(), new _webpackChunkHash2.default(), new _chunkManifestWebpackPlugin2.default({
    filename: jsPath + '/chunk-manifest.js',
    manifestVariable: 'webpackManifest',
    inlineManifest: true
  }), new _uglifyjsWebpackPlugin2.default()] : [new _webpack2.default.HotModuleReplacementPlugin()];

  return [].concat(_toConsumableArray(plugins), environmentPlugins, [

  //  Webpack bundle injection
  //  An array of chunks to be excluded from
  //  the HtmlWebpackPlugin injection.  This is important
  //  because routes that require lazy loading will be
  //  injected into the <head> when they are needed
  new _htmlWebpackPlugin2.default({
    inject: true,
    template: context + '/index.html',
    filename: _path2.default.join(process.cwd(), '.', distOutput, 'index.html'),
    excludeChunks: bundleNames
  }),

  //  Chunk common vendor / main files
  new _webpack2.default.optimize.CommonsChunkPlugin({
    name: ['vendor', 'manifest'],
    minChunks: isVendor
  })], _toConsumableArray(bundleNames.length && bundleNames.map(function (name) {
    return new _webpack2.default.optimize.CommonsChunkPlugin({
      name: name,
      chunks: [name],
      minChunks: 1
    });
  })), [new _extractTextWebpackPlugin2.default({
    filename: isDevEnvironment ? devCssBundle : cssPath + '/' + prodCssBundle,
    allChunks: true
  }),

  //  Output bundles will run in
  //  either production or development
  new _webpack2.default.DefinePlugin({
    'process.env': {
      NODE_ENV: JSON.stringify(isDevEnvironment ? 'development' : 'production')
    }
  })]);
};