import path from 'path';
import webpack from 'webpack';
import postcssImport from 'postcss-import';
import postcssCssNext from 'postcss-cssnext';

export default {
  context: path.resolve(process.cwd(), './src'),
  distOutput: '/public',
  jsPath: 'js',
  cssPath: 'css',
  publicPath: '/assets/',
  indexFile: 'index.html',
  staticAssetsPath: '/public/assets',
  localIdentName: '[name]__[local]',
  autoPrefixer: ['last 2 versions'],
  devCssBundle: 'styles.bundle.css',
  prodCssBundle: 'styles.[contenthash:8].bundle.css',
  diagnostics: {
    heartbeat: '/diagnostic/heartbeat',
    version: '/diagnostic/version',
  },
  isDevEnvironment: (process.env.NODE_ENV !== 'production'),

  //  Loader configs
  babelLoaderConfig: {
    babelrc: false,
    presets: ['es2015', 'react', 'stage-0'],
    plugins: ['transform-regenerator'],
  },
  fileLoaderConfig: {
    name: '[name].[hash].[ext]',
    outputPath: '/images',
  },
  postCssLoaderPlugins: [
    postcssImport({
      addDependencyTo: webpack,
    }),
    postcssCssNext(),
  ],
};
