//  @flow
import path from 'path';
import express from 'express';
import webpackDevMiddleware from 'webpack-dev-middleware';
import webpackHotMiddleware from 'webpack-hot-middleware';
import webpack from 'webpack';

import type { IConfig } from '@/IConfig';
import type { IApp } from '@/IApp';

import { getMessage } from './../../tools/messaging';
import getWebpackConfig from './../webpack/webpack.config';

export default (app: IApp, config: IConfig) => {
  const { isDevEnvironment, publicPath, distOutput, devCssBundle } = config;

  try {
    if (!isDevEnvironment) {
      app.use(publicPath, express.static(path.join(process.cwd(), distOutput, publicPath)));
    }

    if (isDevEnvironment) {
      //  eslint-disable-next-line
      app.locals.compiler = webpack(getWebpackConfig(config));

      //  eslint-disable-next-line
      app.locals.hotMiddleware = webpackHotMiddleware(app.locals.compiler);

      app.locals.compiler.plugin('done', () => {
        const extractedCSS = path.join(process.cwd(), distOutput, publicPath, devCssBundle);
        app.locals.compiler.outputFileSystem.readFile(extractedCSS, (err, data) => {
          if (!err) {
            app.locals.hotMiddleware.publish({ type: 'css-update', value: data.toString() });
          }
        });
      });

      app.use(publicPath, webpackDevMiddleware(app.locals.compiler));
      app.use(app.locals.hotMiddleware);
    }
  } catch (error) {
    throw new Error(getMessage('error', { type: 'static', error }));
  }

  return app;
};
