//  @flow
import fs from 'fs';
import path from 'path';
import type { $Request, $Response } from 'express';
import type { IConfig } from '@/IConfig';
import type { IApp } from '@/IApp';

const resolveHtml = (fileLoader, indexPath) => (
  new Promise((resolve, reject) => (
    fileLoader.readFile(indexPath, (err, fileContents) => {
      let templateString;
      try {
        templateString = fileContents.toString();
      } catch (error) {
        reject('Compilation in progress... Hold tight!');
      } finally {
        resolve(templateString);
      }
    })
  ))
);

export default (app: IApp, config: IConfig) => (
  async (req: $Request, res: $Response, next: Function) => {
    if (app.locals && app.locals.html) {
      next();
    } else {
      const { isDevEnvironment, distOutput, indexFile } = config;
      const indexPath = path.join(process.cwd(), distOutput, indexFile);

      const fileLoader = (isDevEnvironment)
        ? app.locals.compiler.outputFileSystem : fs;

      //  eslint-disable-next-line
      app.locals.html = await resolveHtml(fileLoader, indexPath);
      next();
    }
  }
);
