import { expect } from 'chai';
import { spy } from 'sinon';

import token from './token';

describe('token middleware', () => {
  it('should tokenize the template when environment variables are configured', () => {
    const app = {
      locals: {
        isTokenized: false,
        html: '<div>¡foo!</div>',
      },
    };
    const config = {
      environmentVariables: { foo: 'bar' },
    };

    const middleware = token(app, config);
    const next = spy();

    //  Call the middleware
    //  when express would normally execute it
    middleware({}, {}, next);

    expect(app.locals.isTokenized).to.equal(true);
    expect(app.locals.html).to.equal('<div>bar</div>');
    expect(next.called).to.equal(true);
  });

  it('should return the original template if no tokens match', () => {
    const app = {
      locals: {
        isTokenized: false,
        html: '<div>¡wee!</div>',
      },
    };
    const config = {
      environmentVariables: { foo: 'bar' },
    };

    const middleware = token(app, config);
    const next = spy();

    //  Call the middleware
    //  when express would normally execute it
    middleware({}, {}, next);

    expect(app.locals.isTokenized).to.equal(true);
    expect(app.locals.html).to.equal('<div>¡wee!</div>');
    expect(next.called).to.equal(true);
  });
});
