//  @flow
import ExtractTextPlugin from 'extract-text-webpack-plugin';

import defaultConfig from './../defaultConfig';

import type { IConfig } from '../../flow/IConfig';

export default ({
  localIdentName = defaultConfig.localIdentName,
  babelLoaderConfig = defaultConfig.babelLoaderConfig,
  fileLoaderConfig = defaultConfig.fileLoaderConfig,
  postCssLoaderPlugins = defaultConfig.postCssLoaderPlugins,
}: IConfig) => (
  [
    {
      test: /\.js?$/,
      exclude: /node_modules/,
      loader: 'babel-loader',
      options: { ...babelLoaderConfig },
    },
    {
      test: /\.(css|scss)?$/,
      exclude: /node_modules/,
      loader: ExtractTextPlugin.extract([
        {
          loader: 'css-loader',
          options: {
            modules: true,
            importLoaders: 1,
            localIdentName,
          },
        },
        {
          loader: 'postcss-loader',
          options: {
            plugins: [...postCssLoaderPlugins],
          },
        },
      ]),
    },
    {
      test: /\.(png|svg)?$/,
      exclude: /node_modules/,
      use: {
        loader: 'file-loader',
        options: { ...fileLoaderConfig },
      },
    },
  ]
);
